//program 
* cluster2bootGDLF.ado ---- original cluster2boot.ado written by Mitchell Petersen -- March 2006 --- modified by Ian Gow, Gaizka Ormazabal and Dan Taylor --- July 2009, modified by G. De Luca and G. De Feo --- February 2019
* Program calculates boostrapped clustered standard errors in both a firm and time dimension
* as described by Cameron, Gelbach, and Miller, 2007, "Bootstrap-Based Improvements for Inference with
* Clustered Errors"

#delimit ;
program define cluster2bootGDLF, eclass sortpreserve byable(recall) ;                
	syntax [varlist] [in] [if], fcluster(varname) tcluster(varname) nboot(integer) seed(integer);
	tokenize `varlist';
	local depv `"`1'"';
    local nreps `nboot';
	set seed `seed';

* ---------------------------------------------------------------- ;
* ----- Regression Clustering by First Variable (e.g. Firm) ------ ;
* ---------------------------------------------------------------- ;
	quietly bootstrap "reg `varlist' `in' `if'" _b, reps(`nreps') cluster(`fcluster');
	matrix vcf = e(V);
	local nfcluster=e(N_clust);
* ---------------------------------------------------------------- ;
* ----- Regression Clustering by Second Variable (e.g. Time) ----- ;
* ---------------------------------------------------------------- ;
	quietly bootstrap "reg `varlist' `in' `if'" _b, reps(`nreps') cluster(`tcluster');
	matrix vct = e(V);
	local ntcluster=e(N_clust);
* ---------------------------------------------------------------- ;
* ---------------  Regression with No Clustering  ---------------- ;
* ---------------------------------------------------------------- ;

	   	capture confirm string variable `fcluster';
			if !_rc {;
				gen bc1 = `fcluster'; /* string variable */
		                };
		   	   else {;
		   		gen bc1 = string(`fcluster'); /* numeric */
		   		};
		capture confirm string variable `tcluster';
			if !_rc {;
				gen bc2 = `tcluster'; /* string variable */
		                };
		   	   else {;
		   		gen bc2 = string(`tcluster'); /* numeric */
		   		};
	   	gen bc3 = bc1 + "_" + bc2;
	   	quietly bootstrap "reg `varlist' `in' `if'" _b, reps(`nreps') cluster(bc3);
		drop bc1 bc2 bc3;
	
	matrix coef = e(b);
	matrix vcb = e(V);
	matrix vc = vcf+vct-vcb;

local varlist2: list varlist - depv;
local varlist3 "`varlist2' Constant";

matrix colnames coef = `varlist3';
matrix rownames vc = `varlist3';
matrix colnames vc = `varlist3';

	
* Get degrees of freedom and other basic statistics;

      quietly reg `varlist' `in' `if';
	local nparm = e(df_m)+1;


* ---------------------------------------------------------------- ;
* ----------------- Print out Regression Results ----------------- ;
* ---------------------------------------------------------------- ;
	tokenize `varlist';  		/* this puts varlist in to the macros `1' `2' etc */
	macro shift;			/* drops first arguement (dep var) and shifts the rest up one */
	
	dis " ";
	dis "Linear regression with cluster-bootstrapped SEs";
	dis "Number of obs = " %7.0f e(N);
	dis "Number of clusters (`fcluster') = " _column(31) %5.0f $_nfcluster;
        dis "Number of clusters (`tcluster') = " _column(31) %5.0f $_ntcluster;

	dis " ";
	dis _column(10) "    Beta" _column(24) "  Std Error" 
	    _column(38) "   T-stat" _column(52) "  p-value";
	dis _column(10) " -----------" _column(24) " ------------" 
	    _column(38)	" -----------" _column(52) " -----------";
	
	local i = 1;
	* loop as long as macro `1' is not empty;
	while "`1'" ~= "" {;
		dis "`1'" _column(10) %9.5f coef[1,`i']
			  _column(24) %9.5f sqrt(vc[`i',`i'])
			  _column(38) %9.4f coef[1,`i']/sqrt(vc[`i',`i'])
			  _column(52) %9.4f 2*ttail(e(N)-1,abs(coef[1,`i']/sqrt(vc[`i',`i'])));
		macro shift;
		local i = `i' + 1;
		};
		dis "Constant"
			  _column(10) %9.5f coef[1,`nparm']
			  _column(24) %9.5f sqrt(vc[`nparm',`nparm'])
			  _column(38) %9.4f coef[1,`nparm']/sqrt(vc[`nparm',`nparm'])
			  _column(52) %9.4f 2*ttail(e(N)-1,abs(coef[1,`nparm']/sqrt(vc[`nparm',`nparm'])));
	dis " ";
	dis "     SE clustered by " "`fcluster'" " and " "`tcluster'";
	dis " ";

* ---------------------------------------------------------------- ;
* ----------------- upload Regression Results into e()------------ ;
* ---------------------------------------------------------------- ;

* save statistics from the last regression (clustered by fcluster+tcluster);
* scalars;
	scalar e_N=e(N);
	scalar e_df_m = e(df_m);
	scalar e_df_r = e(df_r);
	scalar e_F = e(F);
	scalar e_r2 = e(r2);
	scalar e_rmse = e(rmse);
	scalar e_mss = e(mss);
	scalar e_rss = e(rss);
	scalar e_r2_a = e(r2_a);
	scalar e_ll = e(ll);
	scalar e_ll_0 = e(ll_0);

* prepare matrices to upload into e();
	ereturn clear;
	tempname b V;
	matrix `b' = coef;
	matrix `V' = vc;

* post the resuls in e();
	ereturn post `b' `V';
	ereturn scalar N = e_N;
	ereturn scalar df_m = e_df_m;
	ereturn scalar df_r = e_df_r;
 	ereturn scalar F= e_F;
	ereturn scalar r2= e_r2;
	ereturn scalar rmse = e_rmse;
	ereturn scalar mss = e_mss;
	ereturn scalar rss = e_rss;
	ereturn scalar r2_a = e_r2_a;
	ereturn scalar ll = e_ll;
	ereturn scalar ll_0 = e_ll_0;
	ereturn local title "Linear regression with cluster-bootstrapped SEs";
	ereturn local method "2-dimension clustered SEs";
	ereturn local depvar "`depv'";
	ereturn local cmd "cluster2";
* end of uploading;


end;
